#if ( !defined(lint))
  static char rcsid[] = "$Header: Set.c,v 1.1 89/11/25 13:44:02 shea Exp $";
#endif

/*  This is the Set widget, it is useful to display a set, without the
 *  overhead of having a widget for each item in the set.  It allows 
 *  the user to select an item in a set and notifies the application through
 *  a callback function.
 *
 *	Created: 	8/13/88
 *	By:		Chris D. Peterson
 *                      MIT - Project Athena
 *	Modified: 	7/17/89
 *	By:		Gary Shea
 *                      UC Berkeley, Dept. of Chemistry
 *	
 */

/***********************************************************
Copyright 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/*
 * Set.c - Set widget
 *
 */

#include <X11/IntrinsicP.h>
#include <stdio.h>
#include <X11/Xos.h>
#include <ctype.h>
#include <X11/StringDefs.h>
#include "SetP.h"

char * malloc();

/* 
 * Default Translation table.
 */

static char defaultTranslations[] =  
   "<Btn1Down>:   Toggle()\n\
   <Btn2Down>:   Toggle()\n\
   <Btn2Up>:   Toggle() NotifyB()";

/****************************************************************
 *
 * Full class record constant
 *
 ****************************************************************/

/* Private Data */

#define offset(field) XtOffset(SetWidget, field)

static XtResource resources[] = {
    {XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
	offset(set.foreground), XtRString, "XtDefaultForeground"},
    {XtNcursor, XtCCursor, XtRCursor, sizeof(Cursor),
       offset(simple.cursor), XtRString, "left_ptr"},
    {XtNfont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
	offset(set.font),XtRString, "XtDefaultFont"},
    {XtNset, XtCSet, XtRPointer, sizeof(XtSetReturnStruct **),
       offset(set.set), XtRString, NULL},
    {XtNdefaultColumns, XtCColumns, XtRInt,  sizeof(int),
	offset(set.default_cols), XtRImmediate, (caddr_t)2},
    {XtNlongest, XtCLongest, XtRInt,  sizeof(int),
	offset(set.longest), XtRImmediate, (caddr_t)0},
    {XtNnumberStrings, XtCNumberStrings, XtRInt,  sizeof(int),
	offset(set.nitems), XtRImmediate, (caddr_t)0},
    {XtNforceColumns, XtCColumns, XtRBoolean,  sizeof(Boolean),
	offset(set.force_cols), XtRString, (caddr_t) "False"},
    {XtNverticalSet, XtCBoolean, XtRBoolean,  sizeof(Boolean),
	offset(set.vertical_cols), XtRString, (caddr_t) "False"},
    {XtNinternalWidth, XtCWidth, XtRDimension,  sizeof(Dimension),
	offset(set.internal_width), XtRImmediate, (caddr_t)4},
    {XtNinternalHeight, XtCHeight, XtRDimension, sizeof(Dimension),
	offset(set.internal_height), XtRImmediate, (caddr_t)2},
    {XtNcolumnSpacing, XtCSpacing, XtRDimension,  sizeof(Dimension),
	offset(set.column_space), XtRImmediate, (caddr_t)6},
    {XtNrowSpacing, XtCSpacing, XtRDimension,  sizeof(Dimension),
	offset(set.row_space), XtRImmediate, (caddr_t)2},
    {XtNcallbackA, XtCCallback, XtRCallback, sizeof(caddr_t),
        offset(set.callbackA), XtRCallback, NULL},
    {XtNcallbackB, XtCCallback, XtRCallback, sizeof(caddr_t),
        offset(set.callbackB), XtRCallback, NULL},
    {XtNcallbackC, XtCCallback, XtRCallback, sizeof(caddr_t),
        offset(set.callbackC), XtRCallback, NULL},
};

static void Initialize();
static void ChangeSize();
static void Resize();
static void Redisplay();
static Boolean Layout();
static XtGeometryResult PreferredGeom();
static Boolean SetValues();
static Boolean ItemInRectangle();
static void FindCornerItems();
static void PaintItemName();
static void HighlightBackground();
static void Notify(), NotifyA(), NotifyB(), NotifyC(), Toggle() ;

static XtActionsRec actions[] = {
      {"NotifyA",         NotifyA},
      {"NotifyB",         NotifyB},
      {"NotifyC",         NotifyC},
      {"Toggle",          Toggle},
      {NULL,NULL}
};

static XtSetDataStruct		noSetSoln ;
static XtSetDataStruct		*noSetSolnPtr = &noSetSoln ;

SetClassRec setClassRec = {
  { /* core_class fields */
    /* superclass	  	*/	(WidgetClass) &simpleClassRec,
    /* class_name	  	*/	"Set",
    /* widget_size	  	*/	sizeof(SetRec),
    /* class_initialize   	*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited       	*/	FALSE,
    /* initialize	  	*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize		  	*/	XtInheritRealize,
    /* actions		  	*/	actions,
    /* num_actions	  	*/	XtNumber(actions),
    /* resources	  	*/	resources,
    /* num_resources	  	*/	XtNumber(resources),
    /* xrm_class	  	*/	NULLQUARK,
    /* compress_motion	  	*/	TRUE,
    /* compress_exposure  	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest	  	*/	FALSE,
    /* destroy		  	*/	NULL,
    /* resize		  	*/	Resize,
    /* expose		  	*/	Redisplay,
    /* set_values	  	*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus	 	*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private   	*/	NULL,
    /* tm_table		   	*/	defaultTranslations,
    /* query_geometry		*/      PreferredGeom,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension          	*/	NULL
  },
  { /* simple_class fields	*/
    /* change_sensitive		*/	XtInheritChangeSensitive
  },
  { /* set_class fields		*/
    /* empty			*/	0
  }
};

WidgetClass setWidgetClass = (WidgetClass)&setClassRec;

/****************************************************************
 *
 * Private Procedures
 *
 ****************************************************************/

static void GetGCs(w)
Widget w;
{
    XGCValues	values;
    SetWidget lw = (SetWidget) w;    

    /* I know this is weird, but i happen to think that white
    *  foregrounds are normal, and my server disagrees...
    */
    values.font		= lw->set.font->fid;
    values.foreground	= lw->core.background_pixel;
    lw->set.normgc = XtGetGC(w, (unsigned) GCFont | GCForeground,
				 &values);

    /* Which implies that reverse video is backward too... */
    values.foreground	= lw->set.foreground;
    lw->set.revgc = XtGetGC(w, (unsigned) GCFont | GCForeground,
				 &values);

    /* This has never been tested, i suspect (as Set, that is). */
    values.foreground = lw->set.foreground;

#ifdef x11r3
    values.tile       = XtGrayPixmap(XtScreen(w));
    values.fill_style = FillTiled;
    lw->set.graygc = XtGetGC(w,
	(unsigned) (GCForeground | GCFont | GCTile | GCFillStyle), &values);
#else
    values.fill_style = FillTiled;
    lw->set.graygc = XtGetGC(w,
	(unsigned) (GCForeground | GCFont | GCFillStyle), &values);
#endif
}

/*	Function Name: ResetSet
 *	Description: Resets the new set when important things change.
 *	Arguments: w - the widget.
 *                 changex, changey - allow the height or width to change?
 *	Returns: none.
 */

static void
ResetSet(w, changex, changey)
Widget w;
Boolean changex, changey;
{
    SetWidget lw = (SetWidget) w;
    int width = w->core.width;
    int height = w->core.height;

    if (lw->set.nitems == 0)	/* Get number of items. */
        while (lw->set.set[lw->set.nitems] != NULL)
	    lw->set.nitems++;

    if (lw->set.longest == 0) { /* Get column width. */
        int i, len, max;
        for ( i = 0, max = 0; i < lw->set.nitems; i++) {
	    len = XTextWidth(lw->set.font, lw->set.set[i]->string,
			     strlen(lw->set.set[i]->string));
	    if (len > max)
	        max = len;
	}
        lw->set.col_width = max;
    }
    else 
        lw->set.col_width = lw->set.longest;
    lw->set.col_width += lw->set.column_space;

    if (Layout(w, changex, changey, &width, &height))
      ChangeSize(w, width, height);
}

/*	Function Name: ChangeSize.
 *	Description: Lays out the widget.
 *	Arguments: w - the widget to try change the size of.
 *	Returns: none.
 */

static void
ChangeSize(w, width, height)
Widget w;
int width, height;
{
    int w_ret, h_ret;

    (void) Layout(w, FALSE, FALSE, &width, &height);

    switch ( XtMakeResizeRequest(w, width, height,(Dimension *) &w_ret,(Dimension *) &h_ret) ) {
    case XtGeometryYes:
    case XtGeometryNo:
        break;
    case XtGeometryAlmost:
        if (Layout(w, FALSE, FALSE, &width, &height)) {
            XtWarning("Size changed when it shouldn't have...");
	    XtWarning("when initializing the Set Widget");
	}
	(void) XtMakeResizeRequest(w, w_ret, h_ret, NULL, NULL);
	break;
    default:
	XtWarning("Unknown geometry return in Set Widget");
	break;
    }
}

/*	Function Name: Initialize
 *	Description: Function that initilizes the widget instance.
 *	Arguments: junk - NOT USED.
 *                 new  - the new widget.
 *	Returns: none
 */

/* ARGSUSED */
static void 
Initialize(junk, new)
Widget junk, new;
{
    SetWidget lw = (SetWidget) new;

/* 
 * Initialize all private resources.
 */

    GetGCs(new);

    /* Set row height. */
    lw->set.row_height = lw->set.font->max_bounds.ascent
			+ lw->set.font->max_bounds.descent
			+ lw->set.row_space;

    ResetSet(new, (new->core.width == 0), (new->core.height == 0));

/*
 * Default to the name of the widget as the entire set.
 * A static XtSetDataStruct kept around for just this purpose
 * is used.
 */

    if (lw->set.set == NULL) {
      lw->set.set = &noSetSolnPtr ;
      noSetSolnPtr->string = lw->core.name ;
      lw->set.nitems = 1;
    }

} /* Initialize */

/*	Function Name: CvtToItem
 *	Description: Converts Xcoord to item number of item containing that
 *                   point.
 *	Arguments: w - the set widget.
 *                 xloc, yloc - x location, and y location.
 *	Returns: the item number.
 */

static int
CvtToItem(w, xloc, yloc, item)
Widget w;
int xloc, yloc;
int *item;
{
    int one, another;
    SetWidget lw = (SetWidget) w;
    int ret_val = OKAY;

    if (lw->set.vertical_cols) {
        one = lw->set.nrows * ((xloc - (int) lw->set.internal_width)
	    / lw->set.col_width);
        another = (yloc - (int) lw->set.internal_height) 
	        / lw->set.row_height;
	 /* If out of range, return minimum possible value. */
	if (another >= lw->set.nrows) {
	    another = lw->set.nrows - 1;
	    ret_val = OUT_OF_RANGE;
	}
    }
    else {
        one = (lw->set.ncols * ((yloc - (int) lw->set.internal_height) 
              / lw->set.row_height)) ;
	/* If in right margin handle things right. */
        another = (xloc - (int) lw->set.internal_width) / lw->set.col_width;
	if (another >= lw->set.ncols) {
	    another = lw->set.ncols - 1; 
	    ret_val = OUT_OF_RANGE;
	}
    }  
    if ((xloc < 0) || (yloc < 0))
        ret_val = OUT_OF_RANGE;
    if (one < 0) one = 0;
    if (another < 0) another = 0;
    *item = one + another;
    if (*item >= lw->set.nitems) return(OUT_OF_RANGE);
    return(ret_val);
}

/*	Function Name: FindCornerItems.
 *	Description: Find the corners of the rectangle in item space.
 *	Arguments: w - the set widget.
 *                 event - the event structure that has the rectangle it it.
 *                 ul_ret, lr_ret - the corners ** RETURNED **.
 *	Returns: none.
 */

static void
FindCornerItems(w, event, ul_ret, lr_ret)
Widget w;
XEvent * event;
int *ul_ret, *lr_ret;
{
    int xloc, yloc;

    xloc = event->xexpose.x;
    yloc = event->xexpose.y;
    CvtToItem(w, xloc, yloc, ul_ret);
    xloc += event->xexpose.width;
    yloc += event->xexpose.height;
    CvtToItem(w, xloc, yloc, lr_ret);
}

/*	Function Name: ItemInRectangle
 *	Description: returns TRUE if the item passed is in the given rectangle.
 *	Arguments: w - the set widget.
 *                 ul, lr - corners of the rectangle in item space.
 *                 item - item to check.
 *	Returns: TRUE if the item passed is in the given rectangle.
 */

static Boolean
ItemInRectangle(w, ul, lr, item)
Widget w;
int ul, lr, item;
{
    SetWidget lw = (SetWidget) w;
    register int mod_item;
    int things;
    
    if (item < ul || item > lr) 
        return(FALSE);
    if (lw->set.vertical_cols)
        things = lw->set.nrows;
    else
        things = lw->set.ncols;

    mod_item = item % things;
    if ( (mod_item >= ul % things) && (mod_item <= lr % things ) )
        return(TRUE);
    return(FALSE);
}

/*	Function Name: HighlightBackground
 *	Description: paints the color of the background for the given item.
 *	Arguments: w - the widget.
 *                 x, y - ul corner of the area item occupies.
 *                 item - the item we are dealing with.
 *                 gc - the gc that is used to paint this rectangle
 *	Returns: none.
 */

static void
HighlightBackground(w, x, y, item, gc)
Widget w;
int x, y, item;
GC gc;
{
    SetWidget lw = (SetWidget) w;
    int hl_x, hl_y, width, height;

    hl_x = x - lw->set.column_space/2;
    width = XTextWidth(lw->set.font, lw->set.set[item]->string,
			 strlen(lw->set.set[item]->string))
		     + lw->set.column_space;
    hl_y = y - lw->set.row_space/2;
    height = lw->set.row_height + lw->set.row_space;

    XFillRectangle(XtDisplay(w), XtWindow(w), gc, hl_x, hl_y, width, height);
}

/*	Function Name: PaintItemName
 *	Description: paints the name of the item in the appropriate location.
 *	Arguments: w - the set widget.
 *                 item - the item to draw.
 *	Returns: none.
 */

static void
PaintItemName(w, item)
Widget w;
int item;
{
    char * str;
    GC gc;
    int x, y, str_y;
    SetWidget lw = (SetWidget) w;
   
    if (lw->set.vertical_cols) {
	x = lw->set.col_width * (item / lw->set.nrows)
	  + lw->set.internal_width;
        y = lw->set.row_height * (item % lw->set.nrows)
	  + lw->set.internal_height;
    }
    else {
        x = lw->set.col_width * (item % lw->set.ncols)
	  + lw->set.internal_width;
        y = lw->set.row_height * (item / lw->set.ncols)
	  + lw->set.internal_height;
    }

    str_y = y + lw->set.font->max_bounds.ascent;

    /* Highly simplified logic for determining highliting...
    *  hope it's not stupid.
    */

    if (lw->set.set[item]->is_lit) {
	gc = lw->set.revgc;
	HighlightBackground(w, x, y, item, lw->set.revgc);
	gc = lw->set.normgc;
    }
    else {
	if (XtIsSensitive(w)) 
	    gc = lw->set.normgc;
	else
	    gc = lw->set.graygc;
	HighlightBackground(w, x, y, item, lw->set.normgc);
	gc = lw->set.revgc;
    }

    str =  lw->set.set[item]->string;	/* draw it */
    XDrawString(XtDisplay(w), XtWindow(w), gc, x, str_y, str, strlen(str));
}
    
/*	Function Name: Redisplay
 *	Description: Repaints the widget window on expose events.
 *	Arguments: w - the set widget.
 *                 event - the expose event for this repaint.
 *                 junk - NOT USED.
 *	Returns: 
 */

/* ARGSUSED */
static void 
Redisplay(w, event, junk)
Widget w;
XEvent *event;
Region junk;
{
    int item;			/* an item to work with. */
    int ul_item, lr_item;       /* corners of items we need to paint. */
    SetWidget lw = (SetWidget) w;

    if (event == NULL) {	/* repaint all. */
        ul_item = 0;
	lr_item = lw->set.nrows * lw->set.ncols - 1;
	XClearWindow(XtDisplay(w), XtWindow(w));
    }
    else
        FindCornerItems(w, event, &ul_item, &lr_item);
    
    for (item = ul_item; (item <= lr_item && item < lw->set.nitems) ; item++)
      if (ItemInRectangle(w, ul_item, lr_item, item))
	PaintItemName(w, item);
}

/*	Function Name: PreferredGeom
 *	Description: This tells the parent what size we would like to be
 *                   given certain constraints.
 *	Arguments: w - the widget.
 *                 intended - what the parent intends to do with us.
 *                 requested - what we want to happen.
 *	Returns: none.
 */

static XtGeometryResult 
PreferredGeom(w, intended, requested)
Widget w;
XtWidgetGeometry *intended, *requested;
{
    int width_req, height_req, new_width, new_height;
    Boolean change;
    
    width_req = intended->request_mode & CWWidth;
    height_req = intended->request_mode & CWHeight;
    if (width_req)
      new_width = intended->width;
    else
      new_width = w->core.width;

    if (height_req)
      new_height = intended->height;
    else
      new_height = w->core.height;

    requested->request_mode = 0;
    
/*
 * We only care about our height and width.
 */

    if ( !width_req && !height_req) {
      return(XtGeometryYes);
    }
    
    change = Layout(w, !width_req, !height_req, &new_width, &new_height);

    requested->request_mode |= CWWidth;
    requested->width = new_width;
    requested->request_mode |= CWHeight;
    requested->height = new_height;

    if (change)
        return(XtGeometryAlmost);
    return(XtGeometryYes);
}

/*	Function Name: Resize
 *	Description: resizes the widget, by changing the number of rows and
 *                   columns.
 *	Arguments: w - the widget.
 *	Returns: none.
 */

static void
Resize(w)
Widget w;
{
  int width, height;

  width = w->core.width;
  height = w->core.height;

  if (Layout(w, FALSE, FALSE, &width, &height))
    XtWarning(
      "Size changed when it shouldn't have in resizing the Set Widget");
}

/*	Function Name: Layout
 *	Description: lays out the item in the set.
 *	Arguments: w - the widget.
 *                 xfree, yfree - TRUE if we are free to resize the widget in
 *                                this direction.
 *                 width, height - the is the current width and height that 
 *                                 we are going to layout the set widget to,
 *                                 depending on xfree and yfree of course.
 *                               
 *	Returns: TRUE if width or height have been changed.
 */

static Boolean
Layout(w, xfree, yfree, width, height)
Widget w;
Boolean xfree, yfree;
int *width, *height;
{
    SetWidget lw = (SetWidget) w;
    Boolean change = FALSE;
    
/* 
 * If force columns is set then always use number of columns specified
 * by default_cols.
 */

    if (lw->set.force_cols) {
        lw->set.ncols = lw->set.default_cols;
	if (lw->set.ncols <= 0) lw->set.ncols = 1;
	/* 12/3 = 4 and 10/3 = 4, but 9/3 = 3 */
	lw->set.nrows = ( ( lw->set.nitems - 1) / lw->set.ncols) + 1 ;
	if (xfree) {		/* If allowed resize width. */
	    *width = lw->set.ncols * lw->set.col_width 
	           + 2 * lw->set.internal_width;
	    change = TRUE;
	}
	if (yfree) {		/* If allowed resize height. */
	    *height = (lw->set.nrows * lw->set.row_height)
                    + 2 * lw->set.internal_height;
	    change = TRUE;
	}
	return(change);
    }

/*
 * If both width and height are free to change the use default_cols
 * to determine the number columns and set new width and height to
 * just fit the window.
 */

    if (xfree && yfree) {
        lw->set.ncols = lw->set.default_cols;
	if (lw->set.ncols <= 0) lw->set.ncols = 1;
	lw->set.nrows = ( ( lw->set.nitems - 1) / lw->set.ncols) + 1 ;
        *width = lw->set.ncols * lw->set.col_width
	       + 2 * lw->set.internal_width;
	*height = (lw->set.nrows * lw->set.row_height)
                + 2 * lw->set.internal_height;
	change = TRUE;
    }
/* 
 * If the width is fixed then use it to determine the number of columns.
 * If the height is free to move (width still fixed) then resize the height
 * of the widget to fit the current set exactly.
 */
    else if (!xfree) {
        lw->set.ncols = (*width - 2 * lw->set.internal_width)
	               / lw->set.col_width;
	if (lw->set.ncols <= 0) lw->set.ncols = 1;
	lw->set.nrows = ( ( lw->set.nitems - 1) / lw->set.ncols) + 1 ;
	if ( yfree ) {
  	    *height = (lw->set.nrows * lw->set.row_height)
		    + 2 * lw->set.internal_height;
	    change = TRUE;
	}
    }
/* 
 * The last case is xfree and !yfree we use the height to determine
 * the number of rows and then set the width to just fit the resulting
 * number of columns.
 */
    else if (!yfree) {		/* xfree must be TRUE. */
        lw->set.nrows = (*height - 2 * lw->set.internal_height) 
	               / lw->set.row_height;
	if (lw->set.nrows <= 0) lw->set.nrows = 1;
	lw->set.ncols = (( lw->set.nitems - 1 ) / lw->set.nrows) + 1;
	*width = lw->set.ncols * lw->set.col_width 
	       + 2 * lw->set.internal_width;
	change = TRUE;
    }      
    return(change);
}

/*	Function Name: Notify
 *	Description: The user callback translations A,B,C each call
 *		   this routine where the callbacks are done.
 *	Arguments: w - the widget that the notify occured in.
 *                 event - event that caused this notification.
 *                 callbackName - The desired user callback.
 *	Returns: none.
 */

/*	Function Name: Notify[ABC]
 *	Description: Calls Notify() with minimal info.
 *	Arguments: w - the widget that the notify occured in.
 *                 event - event that caused this notification.
 *                 params, num_params - not used.
 *	Returns: none.
 */

/* ARGSUSED */
static void
NotifyA(w, event, params, num_params)
Widget w;
XEvent * event;
String * params;
Cardinal *num_params;
{
    Notify ( w, event, XtNcallbackA ) ;
}

/* ARGSUSED */
static void
NotifyB(w, event, params, num_params)
Widget w;
XEvent * event;
String * params;
Cardinal *num_params;
{
    Notify ( w, event, XtNcallbackB ) ;
}

/* ARGSUSED */
static void
NotifyC(w, event, params, num_params)
Widget w;
XEvent * event;
String * params;
Cardinal *num_params;
{
    Notify ( w, event, XtNcallbackC ) ;
}

/*	Function Name: Toggle
 *	Description: Toggle the current element.
 *	Arguments: w - the widget that the event occured in.
 *                 event - event that caused this notification.
 *                 params, num_params - not used.
 *	Returns: none.
 */

/* ARGSUSED */
static void
Toggle(w, event, params, num_params)
Widget w;
XEvent * event;
String * params;
Cardinal *num_params;
{
  SetWidget lw = ( SetWidget ) w;
  int item;

/* Find item and if out of range then return. */

  if ( (CvtToItem(w, event->xbutton.x, event->xbutton.y, &item))
      == OUT_OF_RANGE)
    return;

  XtSetToggle(w, item);	/* Toggle is_lit state of the item. */
}

/*	Function Name: Notify
 *	Description: The user callback translations A,B,C each call
 *		   this routine where the callbacks are done.
 *	Arguments: w - the widget that the notify occured in.
 *                 event - event that caused this notification.
 *                 callbackName - The desired user callback.
 *	Returns: none.
 */

/* ARGSUSED */
static void
Notify(w, event, callbackName)
Widget w;
XEvent * event;
String * callbackName;
{
    SetWidget lw = ( SetWidget ) w;
    int item, item_len;
    XtSetReturnStruct ret_value;

/* 
 * Find item and if out of range then return. 
 */

    if ( ((CvtToItem(w, event->xbutton.x, event->xbutton.y, &item))
	== OUT_OF_RANGE) )
	    return;

/* 
 * Call Callback function.
 */

    ret_value.string = lw->set.set[item]->string;
    ret_value.index = item;
    
    XtCallCallbacks( w,(String) callbackName, (caddr_t) &ret_value);
}

/*
 * Set specified arguments into widget
 */

/* ARGSUSED */
static Boolean SetValues(current, request, new)
Widget current, request, new;
{
    SetWidget cl = (SetWidget) current;
    SetWidget rl = (SetWidget) request;
    SetWidget nl = (SetWidget) new;
    Boolean redraw = FALSE;

    if ((cl->set.foreground != rl->set.foreground) ||
	(cl->core.background_pixel != rl->core.background_pixel) ||
	(cl->set.font != rl->set.font) ) {
        XtDestroyGC(cl->set.normgc);
        XtDestroyGC(cl->set.graygc);
	XtDestroyGC(cl->set.revgc);
        GetGCs(new);
        redraw = TRUE;
    }

    /* Reset row height. */

    if ((cl->set.row_space != rl->set.row_space) ||
	(cl->set.font != rl->set.font)) 
        nl->set.row_height = nl->set.font->max_bounds.ascent
	                    + nl->set.font->max_bounds.descent
			    + nl->set.row_space;
    
    if ((cl->core.width != rl->core.width)                     ||
	(cl->core.height != rl->core.height)                   ||
	(cl->set.internal_width != rl->set.internal_width)   ||
	(cl->set.internal_height != rl->set.internal_height) ||
	(cl->set.column_space != rl->set.column_space)       ||
	(cl->set.row_space != rl->set.row_space)             ||
	(cl->set.default_cols != rl->set.default_cols)       ||
	(  (cl->set.force_cols != rl->set.force_cols) &&
	   (rl->set.force_cols != rl->set.ncols) )           ||
	(cl->set.vertical_cols != rl->set.vertical_cols)     ||
	(cl->set.longest != rl->set.longest)                 ||
	(cl->set.nitems != rl->set.nitems)                   ||
	(cl->set.font != rl->set.font)                       ||
	(cl->set.set != rl->set.set)                        ) {

      ResetSet(new, TRUE, TRUE);
      redraw = TRUE;
    }

    if ((cl->core.sensitive != rl->core.sensitive) ||
	(cl->core.ancestor_sensitive != rl->core.ancestor_sensitive)) {
	redraw = TRUE;
    }
    
    if (!XtIsRealized(current))
      return(FALSE);
      
    return(redraw);
}

/* Exported Functions */

/*	Function Name: XtSetChange.
 *	Description: Changes the set being used and shown.
 *	Arguments: w - the set widget.
 *                 set - the new set.
 *                 nitems - the number of items in the set.
 *                 longest - the length (in Pixels) of the longest element
 *                           in the set.
 *                 resize - if TRUE the the set widget will
 *                          try to resize itself.
 *	Returns: none.
 *      NOTE:      If nitems of longest are <= 0 then they will be calculated.
 *                 If nitems is <= 0 then the set needs to be NULL terminated.
 */

void
XtSetChange(w, set, nitems, longest, resize_it)
Widget w;
XtSetDataStruct ** set;
int nitems, longest;
Boolean resize_it;
{
    SetWidget lw = (SetWidget) w;

    lw->set.set = set;

    if (nitems <= 0) nitems = 0;
    lw->set.nitems = nitems;
    if (longest <= 0) longest = 0;
    lw->set.longest = longest;

    ResetSet(w, resize_it, resize_it);
    if ( XtIsRealized(w) )
      Redisplay(w, NULL, NULL);
}

/*	Function Name: XtSetHighlight
 *	Description: Toggles the given item.
 *	Arguments: w - the set widget.
 *                 item - the item to toggle.
 *	Returns: none.
 */

void
XtSetToggle(w, item)
Widget w;
int item;
{
    SetWidget lw = ( SetWidget ) w;

    if (XtIsSensitive(w)) {
	lw->set.set[item]->is_lit = ! lw->set.set[item]->is_lit ;
	PaintItemName(w, item);
    }
}

