/*  This is the Set widget, it is useful for displaying a set without the
 *  overhead of having a widget for each item in the set.  It allows 
 *  the user to select items in a set and notifies the application through
 *  a callback function.
 *
 *	Created: 	8/13/88
 *	By:		Chris D. Peterson
 *                      MIT - Project Athena
 *	Modified: 	7/17/89
 *	By:		Gary Shea
 *                      UC Berkeley, Dept. of Chemistry
 *	
 *
 */

/***********************************************************
Copyright 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL AND MIT DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#ifndef _XtSet_h
#define _XtSet_h

/***********************************************************************
 *
 * Set Widget
 *
 *
 ***********************************************************************/

#include <X11/Xaw/Simple.h>

/* Resources:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 callbackA           Callback           XtCallbackList  NULL       **6
 callbackB           Callback           XtCallbackList  NULL       **6
 callbackC           Callback           XtCallbackList  NULL       **6
 columnSpacing       Spacing            Dimension       6
 cursor		     Cursor		Cursor		left_ptr
 defaultColumns      Columns            int             2          **5
 destroyCallback     Callback		Pointer		NULL 
 font		     Font		XFontStruct*	XtDefaultFont
 forceColumns        Columns            Boolean         False      **5
 foreground	     Foreground		Pixel		XtDefaultForeground
 height		     Height		Dimension	0          **1
 insensitiveBorder   Insensitive	Pixmap		Gray
 internalHeight	     Height		Dimension	2
 internalWidth	     Width		Dimension	4
 set                 Set                XtSetDataStruct**    NULL       **2
 longest             Longest            int             0          **3  **4
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 numberStrings       NumberStrings      int             0          **4
 pasteBuffer         Boolean            Boolean         False
 rowSpacing          Spacing            Dimension       4
 sensitive	     Sensitive		Boolean		True
 verticalSet         Boolean            Boolean         False
 width		     Width		Dimension	0          **1
 x		     Position		Position	0
 y		     Position		Position	0

 **1 - If the Width or Height of the set widget is zero (0) then the value
       is set to the minimum size necessay to fit the entire set.

       If both Width and Height are zero then they are adjusted to fit the
       entire set that is created width the number of default columns 
       specified in the defaultColumns resource.

 **2 - This is an array of strings the specify elements of the set.
       This resource must be specified. 
       (What good is a set widget without a set??  :-)

 **3 - Longest is the length of the widest string in pixels.

 **4 - If either of these values are zero (0) then the set widget calculates
       the correct value. 

       (This allows you to make startup faster if you already have 
        this information calculated)

       NOTE: If the numberStrings value is zero the set must 
             be NULL terminated.

 **5 - By setting the Set.Columns resource you can force the application to
       have a given number of columns.	     
        
 **6 - This returns the name and index of the item selected in an 
       XtSetReturnStruct that is pointed to by the client_data
       in the CallbackProc.

*/

#define XtCSet                  "Set"
#define XtCSpacing              "Spacing"
#define XtCColumns              "Columns"
#define XtCLongest              "Longest"
#define XtCNumberStrings        "NumberStrings"

#define XtNcolumnSpacing        "columnSpacing"
#define XtNrowSpacing           "rowSpacing"
#define XtNverticalSet          "verticalSet"
#define XtNforceColumns         "forceColumns"
#define XtNnumberStrings        "numberStrings"
#define XtNlongest              "longest"
#define XtNdefaultColumns       "defaultColumns"
#define XtNset                  "set"
#define XtNcallbackA            "callbackA"
#define XtNcallbackB            "callbackB"
#define XtNcallbackC            "callbackC"
 
/* Class record constants */

extern WidgetClass setWidgetClass;

typedef struct _SetClassRec *SetWidgetClass;
typedef struct _SetRec      *SetWidget;

/* The set return structure. */

typedef struct _XtSetReturnStruct {
  String string;
  int index;
} XtSetReturnStruct;

/* The set and toggle structure. */

typedef struct _XtSetDataStruct {
  String string;
  int is_lit;
} XtSetDataStruct;

/******************************************************************
 *
 * Exported Functions
 *
 *****************************************************************/

/*	Function Name: XtSetChange.
 *	Description: Changes the set being used and shown.
 *	Arguments: w - the set widget.
 *                 set - the new set.
 *                 nitems - the number of items in the set.
 *                 longest - the length (in Pixels) of the longest element
 *                           in the set.
 *                 resize - if TRUE the the set widget will
 *                          try to resize itself.
 *	Returns: none.
 *      NOTE:      If nitems of longest are <= 0 then they will be caluculated.
 *                 If nitems is <= 0 then the set needs to be NULL terminated.
 */

extern void XtSetChange(); /* w, set, nitems, longest, resize */

/*	Function Name: XtSetToggle
 *	Description: Toggles state of the given item.
 *	Arguments: w - the set widget.
 *                 item - the item to toggle.
 *	Returns: none.
 */

extern void XtSetToggle(); /* w, item */

#endif /*_XtSet_h */
/* DON'T ADD STUFF AFTER THIS #endif */
