/*  This is the Set widget, it is useful to display a set, without the
 *  overhead of having a widget for each item in the set.  It allows 
 *  the user to select an item in a set and notifies the application through
 *  a callback function.
 *
 *	Created: 	8/13/88
 *	By:		Chris D. Peterson
 *                      MIT - Project Athena
 *	Modified: 	7/17/89
 *	By:		Gary Shea
 *                      UC Berkeley, Dept. of Chemistry
 *	
 * $Header: SetP.h,v 1.1 89/11/25 13:44:15 shea Exp $
 */

/***********************************************************
Copyright 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/* 
 * SetP.h - Private definitions for Set widget
 * 
 */

#ifndef _XtSetP_h
#define _XtSetP_h

/***********************************************************************
 *
 * Set Widget Private Data
 *
 ***********************************************************************/

#include <X11/Xaw/SimpleP.h>
#include "Set.h"

#define OUT_OF_RANGE            -1
#define OKAY                     0

/* New fields for the Set widget class record */

typedef struct {int foo;} SetClassPart;

/* Full class record declaration */
typedef struct _SetClassRec {
    CoreClassPart	core_class;
    SimpleClassPart	simple_class;
    SetClassPart	set_class;
} SetClassRec;

extern SetClassRec setClassRec;

/* New fields for the Set widget record */
typedef struct {
    /* resources */
    Pixel	foreground;
    Dimension	internal_width,
        	internal_height,
                column_space,
                row_space;
    int         default_cols;
    Boolean     force_cols,
                paste,
                vertical_cols;
    int         longest;
    int         nitems;		/* number of items in the set. */
    XFontStruct	*font;
    XtSetDataStruct **set;	/* The set itself. */
    XtCallbackList  callbackA,	/* Give user some options. */
		callbackB,
		callbackC;

    /* private state */

    int         col_width,	/* width of each column. */
                row_height,	/* height of each row. */
                nrows,		/* number of rows in the set. */
                ncols;		/* number of columns in the set. */
    GC		normgc,		/* a couple o' GC's. */
                revgc,
                graygc;		/* used when inactive. */

} SetPart;


/****************************************************************
 *
 * Full instance record declaration
 *
 ****************************************************************/

typedef struct _SetRec {
    CorePart	core;
    SimplePart	simple;
    SetPart	set;
} SetRec;

#endif /*_XtSetP_h*/
