/*
 * xf.h --
 *
 *	Declarations of use to the xf file-utility tool.
 *
 * Copyright 1989 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * Author: Gary Shea, UC Berkeley, Dept. of Chemistry
 *
 * $Header: xf.h,v 1.1 89/11/25 13:44:19 shea Exp $
 */

#ifndef _xf_h
#define _xf_h

/* Generally useful includes. */

# include <stdio.h>
# include <sys/param.h>
# include <sys/types.h>
# include <sys/stat.h>

#ifdef BSD_DIR
#include <sys/dir.h>
#define dirent direct
#else
#include <dirent.h>
#endif

# include <errno.h>
# include <X11/Xlib.h>
# include <X11/Intrinsic.h>
# include <X11/StringDefs.h>
# include <X11/Shell.h>
# include <X11/Xaw/Form.h>
# include <X11/Xaw/Label.h>
# include <X11/Xaw/Dialog.h>
# include <X11/Xaw/Command.h>
# include <X11/cursorfont.h>
# include <X11/Xutil.h>
# include "Set.h"
# include "dir.h"
# include "global.h"

/* data structures */

/* Typing "struct dirent" gets tedious, so here's a typedef: */

typedef struct dirent DirEnt ;

/* procedures */

void	OSFatalError( char *routineName, char *message ) ;
void	UserError( char *message ) ;
void	UserFatalError( char *message ) ;
void	FatalError( char *routineName, char *message ) ;
void	InternalError( char *routineName, char *message ) ;
void	Warning( char *routineName, char *message ) ;

/* System call declarations... */

char	*malloc( unsigned n ) ;

/* Generally useful, non-program-specific definitions... */


# ifndef FALSE
# define FALSE 0
# define TRUE (!FALSE)
# endif

# ifndef ERRLINE_LEN
# define ERRLINE_LEN	300
# endif

/* Some macros... */

# define StrSave( strPtr ) \
    (strcpy( malloc(strlen((strPtr))+1), (strPtr) ))

/* 
 *  From Mark Moraes xlife(1).
 *
 *  This is a simple way to maintain a dynamically allocated array that
 *  grows in chunks. Essentially, obj is a a pointer to the object
 *  (starts off as NULL), and count is the number of elements in it.
 *  size is the currently allocated size, and incr is the size to
 *  increase it by if count == size. type is the type of the object - it
 *  callocs size*sizeof(type) elements for it, and casts this to (type
 *  *). Type is the reason this is a macro and not a procedure.
 *  basically you call this before you store something in obj[count].
 *  Great for adding things at the end of lists.
 */

#define TestAndGrow(obj, count, size, incr, type)\
	if((count) == (size) || (obj) == NULL) { \
		size += (incr);	\
		if ((obj) == NULL) \
			obj = (type *) XtCalloc(size, sizeof(type));\
		else \
			obj = (type *) XtRealloc((char *) obj, \
			 (size)*sizeof(type));\
	} else

#endif /* _xf_h */

